import {
    LimeProperty,
    LimeWebComponent,
    LimeWebComponentContext,
    LimeWebComponentPlatform,
} from '@limetech/lime-web-components';
import { Component, Event, EventEmitter, h, Prop } from '@stencil/core';
import {
    ArticleSelectorComponent,
    CpqArticle,
    CpqItem,
} from 'limepkg-cpq-order-lwc-components';

const colorProperty = 'color';
const pricePerUnitRatio = 2;
const costPriceRatio = 1.2;

type ItemWithColor = CpqItem<{
    [colorProperty]?: string;
}>;

@Component({
    tag: 'example-article-selector-color',
    shadow: true,
})
export class ArticleSelectorColor
    implements LimeWebComponent, ArticleSelectorComponent<ItemWithColor>
{
    @Prop()
    public platform: LimeWebComponentPlatform;

    @Prop()
    public context: LimeWebComponentContext;

    @Prop()
    public articleGroupProperty: LimeProperty;

    @Prop()
    public articleProperty: LimeProperty;

    @Prop()
    public item: ItemWithColor;

    @Event()
    itemChange: EventEmitter<ItemWithColor>;

    private get color() {
        return this.item?.additionalProperties?.[colorProperty];
    }

    private articleOrColorChanged(
        article: CpqArticle | null,
        color: string | null
    ) {
        const item: ItemWithColor = {
            ...this.item,
            article: article,
            description: article?.description,
            pricePerUnit: article?.price,
            vat: article?.vat,
            costPrice: article?.costPrice,
            unit: article?.unit,
            additionalProperties: {
                [colorProperty]: color,
            },
        };

        if (article && color) {
            item.pricePerUnit = article.price * pricePerUnitRatio;
            item.costPrice = article.costPrice * costPriceRatio;
        }

        this.itemChange.emit(item);
    }

    public render() {
        return (
            <coc-article-selector
                platform={this.platform}
                context={this.context}
                item={this.item}
                onArticleChange={this.handleArticleChange}
            >
                <coc-article-group-picker
                    platform={this.platform}
                    context={this.context}
                    label={this.articleGroupProperty.localname}
                    selectedArticleGroup={this.item?.articleGroup}
                />
                <coc-article-picker
                    platform={this.platform}
                    context={this.context}
                    label={this.articleProperty.localname}
                    selectedArticle={this.item?.article}
                    selectedArticleGroup={this.item?.articleGroup}
                />
                <limel-color-picker
                    value={this.color}
                    tooltipLabel="Pick a custom color if you want"
                    helperText="You can also type a color name or value to preview it here"
                    label="Optional article color"
                    onChange={this.handleColorChange}
                />
            </coc-article-selector>
        );
    }

    private handleColorChange = (event: CustomEvent<string>) => {
        event.stopPropagation();

        const color = event.detail;

        this.articleOrColorChanged(this.item?.article, color);
    };

    private handleArticleChange = (event: CustomEvent<CpqArticle | null>) => {
        event.stopPropagation();

        const article = event.detail;

        this.articleOrColorChanged(article, this.color);
    };
}
